using System;
using System.Collections;
using System.IO;
using System.Drawing;

namespace ClassLibrary
{
	/// <summary>
	/// Summary description for ImageDBManager.
	/// </summary>
	public class ImageDBManager
	{
		private ArrayList _images = new ArrayList();
		private DirectoryInfo _di;
		private FileInfo [] _files;

		// lanciato ogni 5% del caricamento totale
		public event EventHandler Progress;

		public int NumFilesInDb
		{
			get{return _files.Length;}
		}

		public ImageDBManager(string path)
		{
			_di = new DirectoryInfo(path);
			_files = _di.GetFiles();
		}

		public void Reset()
		{
			_images = new ArrayList();
		}

		/// <summary>
		/// Carica le immagini da DB
		/// </summary>
		public void LoadAll(int nRegion, int nRGB)
		{
			SetupImages(nRegion, nRGB);
		}

		public IImage [] GetImages()
		{
			IImage [] ims = new IImage [this._images.Count];
			for(int i=0;i<this._images.Count; i++)
				ims[i] = (IImage)this._images[i];
			return ims;
		}


		/// <summary>
		/// Crea n immagini e le memorizza nell'array
		/// </summary>
		/// <param name="dir">la cartella da cui caricare le immagini</param>
		private void SetupImages(int nReg, int nRGB)
		{
			int progressCounter = 0;

			foreach (FileInfo f in _files)
			{
				try 
				{
					ImageImpl2 im = new ImageImpl2(f.FullName,nReg,nRGB);
					im.ComputeRegions();
					this._images.Add(im);
					im.DisposeBitmap();

					// EVENTI

					progressCounter ++;
					// ogni volta che faccio il 5%
					if (progressCounter >= _files.Length/20)
					{
						progressCounter=0;
						if (Progress!=null)
						{
							Progress(this,new EventArgs());
						}
					}
				}
				catch(Exception e){continue;}
			}
			
		}
	}
}
