using System;
using System.Drawing;
using System.Collections;

namespace ClassLibrary
{
	/// <summary>
	/// Summary description for ImageImpl2.
	/// </summary>
	public class ImageImpl2 : IImage
	{
		private static int _staticId = 1;

		private string _bitmapFileName;
		private Bitmap _bitmap;
		private ArrayList _regionList = new ArrayList();

		private int _width = 0;
		private int _height = 0;

		// id di questa istanza
		private int _id = _staticId++;

		/* CAMPI PUBBLICI */

		public string BitmapFileName
		{
			get{return _bitmapFileName;}
		}
		public int Id
		{
			get{return _id;}
		}

		public ArrayList Regions
		{
			get{return _regionList;}
		}

		// lanciato ogni 5% del caricamento
		public event EventHandler Progress;

		public ImageImpl2(string fileName, int nRegions, int nPartizRGB)
		{
			this._bitmapFileName = fileName;
			this._bitmap = new Bitmap(fileName);
			ComputeDimensions();
			CreateRegions (nRegions,nPartizRGB);
//			ComputeRegions();
		}

		#region IImage Members

		public int GetId()
		{
			return this._id;
		}

		public string GetName()
		{
			return this._bitmapFileName;
		}

		public IRegion [] GetRegions()
		{
			IRegion [] i_arr = new IRegion [this._regionList.Count];
			for (int i=0; i< i_arr.Length; i++)
				i_arr[i] = (IRegion)this._regionList[i];
			return i_arr;
		}

		#endregion

		/* Scorre tutti i pixel e li inserisce via via nelle regioni.
		 * Alla fine normalizza tutti i vettori
		 */
		public void ComputeRegions ()
		{
			int progressCounter = 0;
			for (int i=0;i<_width;i++)
			{
				for (int j=0;j<_height;j++)
				{
					Color c = _bitmap.GetPixel(i,j);
					foreach (Region r in this._regionList)
						if (r.Contains(i,j))
							r.AddPixel(c);
				}
				progressCounter++;
				// ogni volta che faccio il 5%
				if (progressCounter >= _width/20)
				{
					progressCounter=0;
					if (Progress!=null)
					{
						Progress(this,new EventArgs());
					}
				}
			}
			NormalizeRegions(10000000);
			PrintRegions();
		}

		public void DisposeBitmap()
		{
		}

		public override string ToString()
		{
			string s = "";
			int i = 1;
            foreach (Region r in this._regionList)
				s+=(i++) + r.ToString()+"\n";
			return s;
		}


	/*****************************************************************************
	 *				UTILITY METHODS
	 * ****************************************************************************/

		private void ComputeDimensions()
		{
			// compute width
			while (true)
			{
				try
				{
					Color c = _bitmap.GetPixel(_width,0);
					_width++;
				}
				catch (Exception e) {break;}
			}

			// compute height
			while (true)
			{
				try
				{
					Color c = _bitmap.GetPixel(0,_height);
					_height ++;
				}
				catch (Exception e) {break;}
			}
		}


		// crea 4 o 5 regioni. (n = numero regioni)
		private void CreateRegions (int n, int nRGB)
		{
//			Console.WriteLine(this._id+": nRegions: "+n);
			int x_meta = this._width/2;
			int y_meta = this._height/2;

			this._regionList.Add(new Region(new Rectangle(0,0,x_meta-1,y_meta-1),nRGB));
			this._regionList.Add(new Region(new Rectangle(x_meta,0,x_meta-1,y_meta-1),nRGB));
			this._regionList.Add(new Region(new Rectangle(0,y_meta,x_meta-1,y_meta-1),nRGB));
			this._regionList.Add(new Region(new Rectangle(x_meta,y_meta,x_meta-1,y_meta-1),nRGB));

			if (n==5)
				this._regionList.Add(new Region(new Rectangle(x_meta/2,y_meta/2,x_meta-1,y_meta-1),nRGB));				
		}

		// normalizza la somma dei valori di ogni vettore ad essere = sum
		private void NormalizeRegions(int sum)
		{
			foreach (Region r in this._regionList)
				r.Normalize(sum);
		}

		private void PrintRegions()
		{
			foreach (Region r in this._regionList)
				Console.WriteLine(this._id+": "+r.ToString());
		}
	}
}
