using System;
using System.Drawing;

namespace ClassLibrary
{
	/// <summary>
	/// Rappresenta una regione
	/// </summary>
	public class Region : IRegion
	{
		private int _nRGB;
		private Rectangle _rectangle;

		// il vettore che rappresenta questa regione
		private int [] _features;
		public int [] Features
		{
			get{return _features;}
		}


		/*
		 * COSTRUTTORE
		 */
		public Region(Rectangle r, int nRGB)
		{	
			this._rectangle = r;
			this._nRGB = nRGB;
			_features = new int[nRGB*nRGB*nRGB];
		}

		/*
		 * INTERFACCIA IREGION
		 */
		public int [] GetArray()
		{
			return this._features;
		}

		/* Aggiunge un elemento al vettore che rappresenta questa regione */
		public void AddPixel(Color c)
		{
			int interval = 255/_nRGB +1;
			int r = c.R/interval;
			int g = c.G/interval;
			int b = c.B/interval;

			int pos = (r*_nRGB*_nRGB)+g*_nRGB +b;
			this._features[pos]++;
		}

		// questa regione contiene il punto x,y?
		public bool Contains (int x, int y)
		{
			return this._rectangle.Contains(x,y);	
		}

		// fa s che la somma degli ele del vettore sia = sum
		public void Normalize (int sum)
		{
			// la somma attuale
			int s = 0;
			for(int i=0;i<this._features.Length;i++)
				s += this._features[i];

			// ciasuno diventa se stesso * sum/s
			for(int i=0;i<this._features.Length;i++)
				this._features[i] *= (sum/s);
		}

		public override string ToString()
		{
			string s = "";
			for (int i=0;i< this._features.Length;i++)
				s += this._features[i]+",";
			return s;
		}

	}
}
