using System;
using System.Drawing;
using System.Windows.Forms;
using System.Collections;

namespace ClassLibrary
{
	/// <summary>
	/// Summary description for Utility.
	/// </summary>
	public class Utility
	{
		public static event EventHandler Progress ;

		public Utility()
		{
		}

		/// <summary>
		/// Calcola l'array di interi corrispondente al bitmap
		/// argomento, se dividiamo ciascuna
		/// regione di colori R,G,B in colorRegions regioni
		/// </summary>
		/// <returns></returns>
		public static int [] ComputeVector(Bitmap MyImage, int nRegions, int colorRegions)
		{
			// inizializza res
			int [] res = new int[colorRegions*colorRegions*colorRegions];
			for (int i=0; i < res.Length; i++)
				res[i] = 0;

			#region compute height and width

			int width = 0;
			int height = 0;

			// compute width
			while (true)
			{
				try
				{
					Color c = MyImage.GetPixel(width,0);
					width++;
				}
				catch (Exception e) {break;}
			}

			// compute height
			while (true)
			{
				try
				{
					Color c = MyImage.GetPixel(0,height);
					height ++;
				}
				catch (Exception e) {break;}
			}

			#endregion

			int progressCounter = 0;
			for (int i=0;i<width;i++)
			{
				for (int j=0;j<height;j++)
				{
					int interval = 255/colorRegions +1;
					int r = MyImage.GetPixel(i,j).R/interval;
					int g = MyImage.GetPixel(i,j).G/interval;
					int b = MyImage.GetPixel(i,j).B/interval;

					int pos = (r*colorRegions*colorRegions)+g*colorRegions +b;
					res [pos]++;
				}
				progressCounter++;
				// ogni volta che faccio il 5%
				if (progressCounter >= width/20)
				{
					progressCounter=0;
					if (Progress!=null)
					{
						Utility.Progress(null,new EventArgs());
					}
				}
				
			}

			return res;
		}

		/// <summary>
		/// Calcola l'array di interi corrispondente al bitmap
		/// argomento, diviso in nRegions regioni e se dividiamo ciascuna
		/// regione di colori R,G,B in colorRegions regioni
		/// </summary>
		/// <returns></returns>
		public static int [,] ComputeMatrix(Bitmap MyImage, int nRegions, int colorRegions)
		{
			ArrayList regions = new ArrayList(nRegions);

			// per ogni regione aggiungi il suo descrittore a regions
			for (int i=0;i<nRegions;i++)
			{
				regions.Add(new int[colorRegions*colorRegions*colorRegions]);
				// azzera il descrittore appena messo
				for (int h=0; h < regions.Count; h++)
					((int[])regions[i])[h] = 0;
			}

			#region compute height and width

			int width = 0;
			int height = 0;

			// compute width
			while (true)
			{
				try
				{
					Color c = MyImage.GetPixel(width,0);
					width++;
				}
				catch (Exception e) {break;}
			}

			// compute height
			while (true)
			{
				try
				{
					Color c = MyImage.GetPixel(0,height);
					height ++;
				}
				catch (Exception e) {break;}
			}

			#endregion

			int progressCounter = 0;
			for (int i=0;i<width;i++)
			{
				for (int j=0;j<height;j++)
				{
					int pixelRegion = 1000;

					#region getPixelRegion
					
					// regione 0
					if (i<width/2 && j<height/2)
						pixelRegion = 0;
					// regione 1
					if (i>width/2 && j<height/2)
						pixelRegion = 1;
					// regione 2
					if (i<width/2 && j>height/2)
						pixelRegion = 2;
					// regione 3
					if (i>width/2 && j>height/2)
						pixelRegion = 3;

					#endregion

					int interval = 255/colorRegions +1;
					int r = MyImage.GetPixel(i,j).R/interval;
					int g = MyImage.GetPixel(i,j).G/interval;
					int b = MyImage.GetPixel(i,j).B/interval;

					int pos = (r*colorRegions*colorRegions)+g*colorRegions +b;
					((int[])regions[pixelRegion]) [pos]++;
				}
				progressCounter++;
				// ogni volta che faccio il 5%
				if (progressCounter >= width/20)
				{
					progressCounter=0;
					if (Progress!=null)
					{
						Utility.Progress(null,new EventArgs());
					}
				}
				
			}

			int [,] mat = new int [colorRegions*colorRegions*colorRegions,
				nRegions];

			#region toMatrix

			for (int i=0;i<width;i++)
			{
				for (int j=0;j<height;j++)
					mat [i,j] = ((int[])regions[i])[j];
			}

			#endregion

			return mat;
		}

	}
}
