using System;
using System.Collections;
using ClassLibrary;

namespace Algorithm
{
	public class Algorithm
	{
		private const int simil = 2000000;		// diecimilioni -> circa il valore medio
		private const int infinito = 10000000; // centomilioni -> molto pi alto della media


		/********************************************************************
		 * AGGIUNTO DA MICHELE
		 * 
		 * PS: Nella cartella di questo progetto c' la dll CLib che esporta
		 * questo metodo
		 */

		[System.Runtime.InteropServices.DllImport("CLib")]
			// la matrice va passata una RIGA dopo l'altra
		private static extern int hungarian (int [] matr, int rows, int cols);
		
		
		// FINE AGGIUNTA
		/******************************************************************/



		// Restituisce tutti gli elementi per cui si  dovuta calcolare la distanza esatta,
		// ogni elemento  formato da un'immagine e dalla distanza che la separa
		// dall'immagine query di riferimento (parametro q).
		public static ElementCollection Compute(IImage q, IImage[] others, int k, int type)
		{
			ElementCollection heap = new ElementCollection();
			ArrayList ub = new ArrayList();
			int UBk = Int32.MaxValue;
			int LB = 0, UB = 0;

			for (int i=0; i < others.Length; i++)
			{
				LB = ComputeLB(q,others[i],type);
				if (LB <= UBk)
				{
					heap.InsertOrder(new Element(others[i],LB));
					UB = ComputeUB(q,others[i],type);
					ub.Add(UB);
					if (ub.Count > k )
					{
						ub.Sort();
						UBk = (int)ub[k];
					}
				}
			}

			ElementCollection result = new ElementCollection();
			int deltaK = Int32.MaxValue;
			int Dist = 0;

			foreach (Element element in heap)
			{
				if (element.Distanza >= deltaK) return result;

				Dist = ComputeDist(q,element.Image,type);
				result.InsertOrder(new Element(element.Image,Dist));
				if (result.Count > k)
				{
					deltaK = (result[k]).Distanza;
				}
			}
			return result;
		}



		/********************************************************************
		 * AGGIUNTA DA MICHELE
		 */

		private static int ComputeDist (IImage q, IImage o, int type)
		{
			// compute distanceMatrix
			int [] array = ComputeDistanceMatrix (q,o,type);

			// formatMatrix
			int lato = Math.Max(q.GetRegions().Length,o.GetRegions().Length);
			int[] squareDistanceMatrix = FormatMatrix(array,q.GetRegions().Length,o.GetRegions().Length,type);
			
			int res = hungarian(squareDistanceMatrix,lato,lato);
			
			return res;

		}



		/* MODIFICATA DA MAURO
		 */
		// Calcola l'upper-bound prendendo il valore pi piccolo di ogni riga
		// ed eliminando la rispettiva colonna per le scelte successive.
		private static int ComputeUB (IImage q, IImage o, int type)
		{
			int minValue;
			int upperBound = 0;
			int minValueCol = 0;
			ArrayList invalideCol = new ArrayList();
			int[] distanceMatrix = ComputeDistanceMatrix(q,o,type);
			int lato = Math.Max(q.GetRegions().Length,o.GetRegions().Length);
			int[] squareDistanceMatrix = FormatMatrix(distanceMatrix,q.GetRegions().Length,o.GetRegions().Length,type);

			for (int i=0; i < lato; i++)
			{
				minValue = Int32.MaxValue;
				for (int j=0; j < lato; j++)
				{
					if ((squareDistanceMatrix[i*lato + j] < minValue) && !invalideCol.Contains(j))
					{
						minValue = squareDistanceMatrix[i*lato + j];
						minValueCol = j;
					}
				}
				upperBound += minValue;
				invalideCol.Add(minValueCol);
			}
			return (upperBound / lato);
		}


		/* MODIFICATA DA MAURO
		 */
		// Calcola il lower-bound prendendo il valore pi piccolo di ogni riga.
		private static int ComputeLB (IImage q, IImage o, int type)
		{
			int minValue;
			int lowerBound = 0;
			int[] distanceMatrix = ComputeDistanceMatrix(q,o,type);
			int lato = Math.Max(q.GetRegions().Length,o.GetRegions().Length);
			int[] squareDistanceMatrix = FormatMatrix(distanceMatrix,q.GetRegions().Length,o.GetRegions().Length,type);

			for (int i=0; i < lato; i++)
			{
				minValue = Int32.MaxValue;
				for (int j=0; j < lato; j++)
				{
					minValue = Math.Min(minValue,squareDistanceMatrix[i*lato + j]);
				}
				lowerBound += minValue;
			}
			return (lowerBound / lato);
		}


//		// Ritorna il minimo valore presente in un array di interi.
//		private static int MinValue(int[] v)
//		{
//			int minimo = Int32.MaxValue;
//
//			for (int k=0; k < v.Length; k++)
//			{
//				minimo = Math.Min(minimo,v[k]);
//			}
//			return minimo;
//		}



		/* Calcola la matrice distanza tra due immagini calcolando per ogni
		 * coppia di vettori la distanza di manhattan.
		 * Il risultato  restituito in forma di array in cui ci sono le righe
		 * una dopo l'altra
		 */
		private static int [] ComputeDistanceMatrix (IImage a, IImage b, int type)
		{
			// L'array risultato  lungo il prodotto del n.ro di regioni di a e b
			int [] res = new int[a.GetRegions().Length * b.GetRegions().Length];

			for (int i=0;i<a.GetRegions().Length; i++)
				for (int j=0; j<b.GetRegions().Length; j++)
				{
					IRegion ra = a.GetRegions()[i];
					IRegion rb = b.GetRegions()[j];
					res [i*b.GetRegions().Length + j] = ComputeRegionDistance (ra,rb);
				}
			return res;
		}



		/**********************************************************************
		 * AGGIUNTO DA MAURO
		 * 
		 */

		// Modifica la matrice (rappresentata come array) in modo da renderla quadrata
		// aggiungendo dei pesi alle righe o alle colonne il cui valore dipende dal tipo
		// della query stessa (Contains:  p1 = 0, p2 > 0;	 Part-of:  p1 > 0, p2 = 0).
		private static int[] FormatMatrix(int[] matrix, int righe, int colonne, int type)
		{
			if (righe == colonne) return matrix;

			int lato = Math.Max(righe,colonne);
			int[] squareMatrix = new int[lato*lato];

			// aggiungo delle righe (p1)
			if (righe < colonne)
			{
				for (int i=0; i < matrix.Length; i++)
				{
					squareMatrix[i] = matrix[i];
				}
				for (int j=matrix.Length; j < squareMatrix.Length; j++)
				{
					switch (type)
					{
						case 0 : squareMatrix[j] = simil;		// similarity
							break;
						case 1 : squareMatrix[j] = infinito;	// part-of
							break;
						case 2 : squareMatrix[j] = 0;			// contains
							break;
					}
				}
				return squareMatrix;
			}

			// aggiungo delle colonne (p2)
			else if (righe > colonne)
			{
				for (int r=0; r < righe; r++)
				{
					for (int c=0; c < colonne; c++)
					{
						squareMatrix[r*righe + c] = matrix[r*colonne + c];
					}
					for (int k=colonne; k < righe; k++)
					{
						switch (type)
						{
							case 0 : squareMatrix[r*righe + k] = simil;		// similarity
								break;
							case 1 : squareMatrix[r*righe + k] = 0;			// part-of
								break;
							case 2 : squareMatrix[r*righe + k] = infinito;	// contains
								break;
						}
					}
				}
				return squareMatrix;
			}

			return squareMatrix;
		}

		/* FINE AGGIUNTA (MAURO) */
		/****************************************************************/
	


		// calcola la distanza tra 2 regioni
		private static int ComputeRegionDistance (IRegion r1, IRegion r2)
		{
			return ComputeVectorDistance (r1.GetArray(), r2.GetArray());
		}


		// calcola la distanza di Manhattan di due array
		private static int ComputeVectorDistance (int [] a, int [] b)
		{
			// precondizione
			if (a.Length != b.Length)
				throw new Exception("I due array hanno misura diversa");

			// corpo del metodo
			int sum = 0;
			for (int i=0; i<a.Length; i++)
				sum += Math.Abs(a[i] - b[i]);

			return sum;
		}


		/* FINE AGGIUNTA
		********************************************************************/
	}
}
