using System;
using System.Net.Sockets;
using System.Net;

namespace ClassLibrary
{
	/// <summary>
	/// Summary description for Utility.
	/// </summary>
	public class SocketUtility
	{
		/// <summary>
		/// Crea un socket per comunicazioni TCP standard
		/// </summary>
		/// <returns>La socket creata</returns>
		public static Socket CreateSocket()
		{
			Socket socket = new Socket(AddressFamily.InterNetwork,SocketType.Stream,
			ProtocolType.Tcp);
			socket.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.SendTimeout,2000);
			socket.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReceiveTimeout, 2000);
			return socket;
		}

		/// <summary>
		/// Connette la socket ad un server locale che  presente
		/// sulla porta data
		/// </summary>
		/// <param name="socket">la socket da connettere</param>
		/// <param name="serverPort">la porta del server locale</param>
		public static void ConnectToLocalServer (Socket socket, int serverPort)
		{
			socket.Connect(new IPEndPoint(IPAddress.Parse("127.0.0.1"),serverPort));
		}

		/// <summary>
		/// Invia un messaggio su una socket
		/// </summary>
		/// <param name="message"></param>
		public static void Send (Socket socket, string message)
		{
			byte[] msg = System.Text.Encoding.ASCII.GetBytes(message);
			int i = socket.Send(msg);
		}

		/// <summary>
		/// Riceve una stringa da socket. La stringa
		/// attesa  della forma ' *_ ', nel senso
		/// che il carattere underscore mi rappresenta
		/// l'ultimo carattere ricevuto
		/// 
		/// </summary>
		/// <param name="socket">la socket da cui ricevo</param>
		/// <returns>la stringa ricevuta</returns>
		public static string Receive (Socket socket)
		{
			byte[] bytes = new byte[1024];
			socket.Receive(bytes);
			string ricevuto = System.Text.Encoding.ASCII.GetString(bytes);
			string ritorno = TrimUnderscore(ricevuto);
			//			Console.WriteLine("Ritorno:"+ritorno);
			return ritorno;
		}

		// underscore rappresenta l'ultimo carattere
		private static string TrimUnderscore(string s)
		{
			string parola = "";
			char [] arr = s.ToCharArray();
			foreach (char c in arr)
			{
				if (c!='_')
					parola=parola+c;
				else
					return parola;
			}
			return parola;
		}

		/*
		 * UTILIZZO TIPICO
		 * 
		 * 
		 * 
		 * 
		[Test]
		public void BottaRispostaTest()
		{
			Socket socket = Utility.CreateSocket();
			socket.Connect(new IPEndPoint(IPAddress.Parse("127.0.0.1"),serverPort));
			Conversazione conv = new Conversazione(socket);
			Console.WriteLine("BottaRispostaTest:converso con "+
				"IP:"+conv.RemoteIpEndPoint.Address+"."+
				"porta:"+conv.RemoteIpEndPoint.Port);
			conv.Send("myMes");
			string ricevuto = conv.Receive();
			socket.Close();
			Assertion.Assert(ricevuto == "SimpleHandlerMessage");
		}
*/

	}
}
